! =================================================================================================================================
! MODULE       : lpj_gap
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Simulate mortality of individuals and update biomass, litter and 
!! stand density of the PFT
!!
!!\n DESCRIPTION : Simulate mortality of individuals and update biomass, litter and 
!! stand density of the PFT. This module differs from lpj_kill.f90 in that this 
!! module kills individuals within a PFT and lpj_kill.f90 removes a PFT from a 
!! gridbox
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S) : 
!! - Sitch, S., B. Smith, et al. (2003), Evaluation of ecosystem dynamics,
!!         plant geography and terrestrial carbon cycling in the LPJ dynamic 
!!         global vegetation model, Global Change Biology, 9, 161-185.\n
!! - Waring, R. H. (1983). "Estimating forest growth and efficiency in relation 
!!         to canopy leaf area." Advances in Ecological Research 13: 327-354.\n
!! 
!! SVN          :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================

MODULE lpj_gap

  ! modules used:
  USE xios_orchidee
  USE stomate_data
  USE pft_parameters
  USE ioipsl_para 
  USE constantes

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC gap_diagnostic
  
  ! Variable declaration 

  LOGICAL, SAVE             :: firstcall = .TRUE.                !! first call flag
!$OMP THREADPRIVATE(firstcall)

CONTAINS


!! ================================================================================================================================
!! SUBROUTINE   : gap_diagnostic
!!
!>\BRIEF        Simulate tree and grass mortality, transfer dead biomass to litter and update stand density
!!
!! DESCRIPTION  : Calculate mortality of trees and grasses, transfer the dead biomass to litter pool,
!! and update biomass pool and number of individuals. To get tree mortality, it's possible to choose either a 
!! constant mortality rate; or to calculate the tree mortality rate based on it's growth efficiency, which is 
!! defined as this year's net biomass increment per leaf area.\n
!!
!! When using growth efficiency mortality, first calculate the net biomass increment for the last year, then 
!! calculate the growth efficiency and finally calculate the growth efficiency mortality.\n
!!
!! Eqation to calculate growth efficiency:
!! \latexonly 
!!     \input{gap1.tex}
!! \endlatexonly
!! Where $greff$ is growth efficiency, $\Delta$ is net biomass increment, 
!! $C_{leaf} is last year's leaf biomass, and $SLA$ the specific leaf area. 
!! 
!!
!! Eqation to calculate growth efficiency mortality:
!! \latexonly 
!!     \input{gap2.tex}
!! \endlatexonly
!! Where $mort_{greff}$ is the growth efficiency mortality, $greff$ is growth  
!! efficiency, $k_{mort1}$ is asymptotic maximum mortality rate.
!! 
!! The name for variable ::availability is not well chosen. Actually the meaning of the variable is mortailty 
!! rate derived from growth efficiency related mortality. ?? Suggestion: change the name "availability" to 
!! "mortgref", which means "mortality caused by ".\n
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): ::biomass; biomass, ::ind density of individuals, ::bm_to_litter biomass transfer 
!! to litter and ::mortality mortality (fraction of trees that is dying per time step)
!!
!! REFERENCE(S)   :
!! - Sitch, S., B. Smith, et al. (2003), Evaluation of ecosystem dynamics,
!!         plant geography and terrestrial carbon cycling in the LPJ dynamic 
!!         global vegetation model, Global Change Biology, 9, 161-185.
!! - Waring, R. H. (1983). "Estimating forest growth and efficiency in relation 
!!         to canopy leaf area." Advances in Ecological Research 13: 327-354.
!!
!! FLOWCHART    : None
!!\n
!_ ================================================================================================================================
  
  SUBROUTINE gap_diagnostic (npts, dt, npp_longterm, turnover_longterm, lm_lastyearmax, &
       PFTpresent, biomass, ind, bm_to_litter, mortality, &
       forest_managed)

    !! 0. Variable and parameter declaration

    !! 0.1 Input variables 
    INTEGER(i_std), INTENT(in)                              :: npts                    !! Domain size (-)
    REAL(r_std), INTENT(in)                                 :: dt                      !! Time step (days)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)            :: npp_longterm            !! "Long term" (default 3-year) net primary
                                                                                       !! productivity  
                                                                                       !! @tex $(gC m^{-2} year^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(in) :: turnover_longterm !! "Long term" (default 3-year) turnover  
                                                                                       !! rate @tex $(gC m^{-2} year^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)            :: lm_lastyearmax          !! Last year's maximum leaf mass, for each 
                                                                                       !! PFT @tex $(gC m^{-2})$ @endtex
    LOGICAL, DIMENSION(npts,nvm), INTENT(in)                :: PFTpresent              !! Is the pft present in the pixel
    INTEGER(i_std), DIMENSION (npts,nvm), INTENT(in)        :: forest_managed          !! forest management flag

    !! 0.2 Output variables

    REAL(r_std), DIMENSION(npts,nvm),INTENT(out)            :: mortality               !! Mortality (fraction of trees that is 
                                                                                       !! dying per time step)

    !! 0.3 Modified variables   
    
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(inout)  :: biomass       !! Biomass @tex $(gC m^{-2}) $@endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)                   :: ind           !! Number of individuals 
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(inout)  :: bm_to_litter  !! Biomass transfer to litter 
                                                                                       !! @tex $(gC m^{-2})$ @endtex  

    !! 0.4 Local variables

    REAL(r_std), DIMENSION(npts)                            :: delta_biomass           !! Net biomass increase for the previous 
                                                                                       !! year @tex $(gC m^{-2} year^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts)                            :: dmortality              !! Dead biomass caused by mortality 
                                                                                       !! @tex $(gC m^{-2}) $@endtex
    REAL(r_std), DIMENSION(npts)                            :: vigour                  !! Growth efficiency, an indicator of tree 
                                                                                       !! vitality, used to calculate mortality
    REAL(r_std), DIMENSION(npts)                            :: availability            !! Mortality rate derived by growth 
                                                                                       !! efficiency @tex $(year^{-1})$ @endtex 
    INTEGER(i_std)                                          :: j,k,m,i                 !! Indices

!_ ================================================================================================================================

    IF ( firstcall ) THEN

       firstcall = .FALSE.

    ENDIF

    IF (bavard.GE.3) WRITE(numout,*) 'Entering gap',control%ok_constant_mortality

    mortality(:,:) = zero

    DO j = 2,nvm  ! loop over #PFT

       !! 1. Tree mortality

       IF ( is_tree(j) ) THEN 

          DO  i = 1,npts ! loop over land_points

             IF ( forest_managed(i,j) == 0 ) THEN

                !! 1.1 Use growth efficiency or constant mortality?
                IF ( .NOT.  control%ok_constant_mortality  ) THEN

                   !! 1.1.1 Estimate net biomass increment
                   !  To calculate growth efficiency mortality, first estimate net biomass increment by 
                   !  subtracting turnover from last year's NPP.
                   IF ( PFTpresent(i,j) .AND. ( lm_lastyearmax(i,j) .GT. min_stomate ) ) THEN

                      ! note that npp_longterm is now actually longterm growth efficiency (NPP/LAI)
                      ! to be fair to deciduous trees calculate net biomass increment
                      delta_biomass(i) = MAX( npp_longterm(i,j) - ( turnover_longterm(i,j,ileaf,icarbon) + &
                           turnover_longterm(i,j,iroot,icarbon) + turnover_longterm(i,j,ifruit,icarbon) + & 
                           turnover_longterm(i,j,isapabove,icarbon) + turnover_longterm(i,j,isapbelow,icarbon) ), zero)

                      !! 1.1.2 Calculate growth efficiency 
                      !  Calculate growth efficiency by dividing net biomass increment by last year's 
                      !  maximum LAI. (corresponding actually to the maximum LAI of the previous year)
                      vigour(i) = delta_biomass(i) / (lm_lastyearmax(i,j)*sla(j))

                   ELSE

                      vigour(i) = zero

                   END IF ! PFTpresent(i,j) .AND. ( lm_lastyearmax(i,j) .GT. min_stomate)

                   !! 1.1.3 Calculate growth efficiency mortality rate
                   IF ( PFTpresent(i,j) ) THEN

                      availability(i) = availability_fact / ( un + ref_greff * vigour(i) )

                      ! Scale mortality by timesteps per year
                      mortality(i,j) = MAX(min_avail, availability(i))  * dt/one_year  

                   END IF ! PFTpresent(i,j)

                ELSE  ! .NOT. control%ok_constant_mortality

                   !! 1.2 Use constant mortality 
                   !  constant mortality accounts for the residence time of each tree PFT
                   IF ( PFTpresent(i,j) ) THEN

                      mortality(i,j) = dt/(residence_time(j)*one_year)

                   END IF ! PFTpresent(i,j)

                ENDIF ! .NOT.  control%ok_constant_mortality

                !! 1.3 Mortality in DGVM
                !  If the long term NPP is zero, all trees are killed
                !  +++CHECK+++
                !  This is this only applied in the DGVM maybe in order to make 
                !  the DGVM respond faster and thus make the vegetation dynamics 
                !  more dynamic? The link with lpj_kill.f90 is still not clear 
                !  and should be checked by who is working on the DGVM. 
                IF ( control%ok_dgvm ) THEN

                   IF ( PFTpresent(i,j) .AND. ( npp_longterm(i,j) .LE. min_stomate ) ) THEN

                      mortality(i,j) = un

                   END IF ! PFTpresent(i,j) .AND. ( npp_longterm(i,j) .LE. min_stomate

                ENDIF
                !  +++++++++++

                !! 1.4 Update biomass and litter pools 
                !  Update biomass and litter pool after dying and transfer recently died biomass to litter
                DO m = 1,nelements

                   DO k = 1, nparts

                      IF ( PFTpresent(i,j) ) THEN

                         dmortality(i) =  mortality(i,j) * biomass(i,j,k,m)
                         bm_to_litter(i,j,k,m) = bm_to_litter(i,j,k,m) + dmortality(i)
                         biomass(i,j,k,m) = biomass(i,j,k,m) - dmortality(i)

                      END IF ! PFTpresent(i,j) 

                   ENDDO !nparts

                END DO !nelements

                !! 1.5 In case of dynamic vegetation, update tree individual density
                IF ( control%ok_dgvm .OR. .NOT.control%ok_constant_mortality) THEN

                   IF ( PFTpresent(i,j) ) THEN

                      ind(i,j) = ind(i,j) * ( un - mortality(i,j) )

                   END IF ! PFTpresent(i,j)

                ENDIF !( control%ok_dgvm .OR. .NOT.control%ok_constant_mortality)

             END IF ! (forest_managed(i,j) == 0)

          END DO ! loop over land points


       !! 2. Grasses mortality
       ELSE 

          ! For grasses, if last year's NPP is very small (less than 10 gCm^{-2}year{-1})
          ! the grasses completely die
          IF ( .NOT.control%ok_dgvm .AND. .NOT.control%ok_constant_mortality) THEN

             DO  i = 1,npts ! loop over land_points

                IF ( PFTpresent(i,j) .AND. &
                     ( npp_longterm(i,j) .LE. npp_longterm_init ) ) THEN

                   mortality = zero

                END IF

                ! Update biomass and litter pools
                DO m = 1,nelements

                   DO k = 1, nparts

                      IF ( PFTpresent(i,j) ) THEN

                         dmortality(i) =  mortality(i,j) * biomass(i,j,k,m)
                         bm_to_litter(i,j,k,m) = bm_to_litter(i,j,k,m) + dmortality(i)
                         biomass(i,j,k,m) = biomass(i,j,k,m) - dmortality(i)

                      END IF ! PFTpresent(i,j)

                   ENDDO ! nparts

                END DO !nelements

             END DO ! loop over land points

          ENDIF ! .NOT.control%ok_dgvm .AND. .NOT.control%ok_constant_mortality

       ENDIF   !IF ( is_tree(j) )

    ENDDO      !loop over pfts


    !! 3. Write to history files
       
    ! output in fraction of trees that dies/day.
    mortality(:,:) = mortality(:,:) / dt
    
    CALL xios_orchidee_send_field("MORTALITY",mortality)

    CALL histwrite_p (hist_id_stomate, 'MORTALITY', itime, &
         mortality, npts*nvm, horipft_index)
    
    IF (bavard.GE.4) WRITE(numout,*) 'Leaving gap'
       
  END SUBROUTINE gap_diagnostic


END MODULE lpj_gap
