! =================================================================================================================================
! MODULE       : stomate_kill
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Simulate mortality of individuals and update biomass, litter and 
!! stand density of the PFT
!!
!!\n DESCRIPTION : Simulate mortality of individuals and update biomass, litter and 
!! stand density of the PFT. This module replaces lpj_gap/kill.  Biomass actually
!! dies here and is moved to the litter pools.  This does not work with the DGVM
!! at the moment.
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S) : 
!! - Sitch, S., B. Smith, et al. (2003), Evaluation of ecosystem dynamics,
!!         plant geography and terrestrial carbon cycling in the LPJ dynamic 
!!         global vegetation model, Global Change Biology, 9, 161-185.\n
!! - Waring, R. H. (1983). "Estimating forest growth and efficiency in relation 
!!         to canopy leaf area." Advances in Ecological Research 13: 327-354.\n
!! 
!! SVN          :
!! $HeadURL: svn://forge.ipsl.jussieu.fr/orchidee/branches/ORCHIDEE-DOFOCO/ORCHIDEE/src_stomate/stomate_kill.f90 $
!! $Date: 2013-09-27 09:59:24 +0200 (Fri, 27 Sep 2013) $
!! $Revision: 1485 $
!! \n
!_ ================================================================================================================================

MODULE stomate_kill

  ! modules used:

  USE stomate_data
  USE pft_parameters
  USE ioipsl_para 
  USE function_library,       ONLY : distribute_mortality_biomass,&
                                     nmax,  wood_to_dia
  USE constantes
  USE stomate_prescribe
  USE sapiens_lcchange,       ONLY : merge_biomass_pfts
  USE stomate_prescribe
!  USE lpj_kill 

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC gap_prognostic, gap_clear, gap_clean
  
  ! Variable declaration 

  LOGICAL, SAVE             :: firstcall = .TRUE.                !! first call flag
!$OMP THREADPRIVATE(firstcall)

CONTAINS


!! ================================================================================================================================
!! SUBROUTINE   : gap_clear
!!
!>\BRIEF        Set the firstcall flag back to .TRUE. to prepare for the next simulation.
!_ ================================================================================================================================
  
  SUBROUTINE gap_clear
     firstcall = .TRUE.
  END SUBROUTINE gap_clear


!! ================================================================================================================================
!! SUBROUTINE   : gap_prognostic
!!
!>\BRIEF        Transfer dead biomass to litter and update stand density for trees
!!              which die of natural causes.
!!
!! DESCRIPTION  : This routine has a simple purpose: kill individuals by natural causes.  
!!   The variable circ_class_kill indicates how many individuals need to be killed by the various
!!   processes in the code, and is calculated in other modules.  gap_prognostic takes
!!   this variable and decides on the correct order of which to actually kill the
!!   trees, making sure that we don't double count.  
!!
!! RECENT CHANGE(S): 
!!
!! MAIN OUTPUT VARIABLE(S): ::circ_class_biomass; biomass, ::ind density of individuals, 
!! ::mortality mortality (fraction of 
!! trees that is dying per time step)
!!
!! REFERENCE(S)   :
!! - Sitch, S., B. Smith, et al. (2003), Evaluation of ecosystem dynamics,
!!         plant geography and terrestrial carbon cycling in the LPJ dynamic 
!!         global vegetation model, Global Change Biology, 9, 161-185.
!! - Waring, R. H. (1983). "Estimating forest growth and efficiency in relation 
!!         to canopy leaf area." Advances in Ecological Research 13: 327-354.
!!
!! FLOWCHART    : None
!!\n
!_ ================================================================================================================================
 
  SUBROUTINE gap_prognostic (npts, biomass, ind,  bm_to_litter, &
       circ_class_biomass, circ_class_kill, circ_class_n, &
       veget_max, circ_class_cwd, circ_class_leafcwd)

    !! 0. Variable and parameter declaration

    !! 0.1 Input variables 
    INTEGER(i_std), INTENT(in)                                   :: npts               !! Domain size (-)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)                   :: veget_max          !! Maximum fraction of vegetation type 

    !! 0.2 Output variables

    !! 0.3 Modified variables   
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                             INTENT(inout)       :: biomass            !! Biomass @tex $(gC m^{-2}) $@endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)              :: ind                !! Stand level number of individuals 
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements), &                         
                                              INTENT(inout)      :: circ_class_biomass !! Biomass of the components of the model  
                                                                                       !! tree within a circumference
                                                                                       !! class @tex $(gC ind^{-1})$ @endtex  
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(inout)        :: circ_class_n       !! Number of individuals in each circ class
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nfm_types,ncut_times),&
                                                  INTENT(inout)  :: circ_class_kill    !! Number of trees within a circ that needs
   REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(inout)        :: circ_class_cwd                    
                                                                 !! to be killed @tex $(ind m^{-2})$ @endtex
        REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(inout)        :: circ_class_leafcwd    
REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                                  INTENT(inout)  :: bm_to_litter       !! Biomass transfer to litter 
                                                                                       !! @tex $(gC m^{-2})$ @endtex

    !! 0.4 Local variables
    INTEGER(i_std)                                               :: ipts, ivm, ipar    !! Indices
    INTEGER(i_std)                                               :: iele, icir, imbc   !! Indices
    INTEGER(i_std)                                               :: ifm,icut           !! Indices
    INTEGER,DIMENSION(nvm)                                       :: nkilled            !! the number of grid points at which a given
                                                                                       !! given PFT's biomass has been reduced to zero
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements)           :: check_intern       !! Contains the components of the internal
                                                                                       !! mass balance chech for this routine
                                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                   :: closure_intern     !! Check closure of internal mass balance
                                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                   :: pool_start         !! Start pool of this routine 
                                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                   :: pool_end           !! End pool of this routine 
                                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex

!_ ================================================================================================================================

    !! 1. Set firstcall flag

    IF ( firstcall ) THEN

       firstcall = .FALSE.

    ENDIF

    IF (bavard.GE.2) WRITE(numout,*) 'Entering gap. Use constant mortality = ', &
         control%ok_constant_mortality

    !! 2. Initialize variables
    nkilled(:)=0
    
    ! 2.1 Initialize check for mass balance closure
    !  The mass balance is calculated at the end of this routine
    !  in section .
    pool_start = zero
    DO ipar = 1,nparts
       DO iele = 1,nelements
          !  Initial litter and biomass
          pool_start(:,:,iele) = pool_start(:,:,iele) + &
               (biomass(:,:,ipar,iele) * veget_max(:,:)) + &
               (bm_to_litter(:,:,ipar,iele) * veget_max(:,:))
       ENDDO
    ENDDO


 !! 3. Kill plants
    DO  ipts = 1,npts ! loop over land_points

       DO ivm = 2,nvm  ! loop over #PFT


          IF(veget_max(ipts,ivm) == zero)THEN
              ! this vegetation type is not present, so no reason to do the 
              ! calculation
               CYCLE
          ENDIF


          !! First, all the plants that are supposed to be killed by fire are killed.
          !! We do not yet know how to handle this, so this will have to be taken
          !! into account when SPITFIRE is coupled.

          DO iele = 1,nelements

             !! All of the natural death is grouped 
             !! together in one pool since all of the biomass will be left on 
             !! site (moved to the litter pools).

             ! Killing is done a little differently for trees and non-trees, since
             ! circ_classes are not defined for non-trees.

             DO ifm=1,nfm_types
                DO icut=1,ncut_times

                   ! Since we gave circ_class_kill the same dimensions as the
                   ! harvest pool, we need to explicitly say which combinations
                   ! of ifm and icut are killed in which ways.  Currently, I am
                   ! putting all natural death into ifm_none, even if it happens
                   ! in another management type (for example, self-thinning will 
                   ! happen with ifm_thin, but it's really a natural death).
                   IF(ifm .NE. ifm_none)CYCLE
                   IF((icut .NE. icut_thin) .AND. (icut .NE. icut_clear))CYCLE

                   IF(is_tree(ivm))THEN

                      DO icir=1,ncirc

                         IF(test_pft == ivm .AND. test_grid == ipts .AND. ld_kill)&
                              WRITE(numout,*) 'killing before: ',ipts,ivm,icir,ifm,icut,&
                              circ_class_kill(ipts,ivm,icir,ifm,icut),circ_class_n(ipts,ivm,icir)

                         ! move the dead biomass to the respective litter pool

                         bm_to_litter(ipts,ivm,:,iele) = bm_to_litter(ipts,ivm,:,iele) + &
                              circ_class_biomass(ipts,ivm,icir,:,iele)*&
                              circ_class_kill(ipts,ivm,icir,ifm,icut)
                         ! litter in each circ
                        circ_class_cwd(ipts,ivm,icir)=(circ_class_biomass(ipts,ivm,icir,1,iele)+circ_class_biomass(ipts,ivm,icir,2,iele)+circ_class_biomass(ipts,ivm,icir,4,iele)+circ_class_biomass(ipts,ivm,icir,7,iele))*circ_class_kill(ipts,ivm,icir,ifm,icut)
                        circ_class_leafcwd(ipts,ivm,icir)=circ_class_biomass(ipts,ivm,icir,1,iele)*circ_class_kill(ipts,ivm,icir,ifm,icut)
                         ! remove the number of individuals that died
                         circ_class_n(ipts,ivm,icir) = circ_class_n(ipts,ivm,icir) - &
                              circ_class_kill(ipts,ivm,icir,ifm,icut)


                         ! remove the dead biomass
                         biomass(ipts,ivm,:,iele) = biomass(ipts,ivm,:,iele) - &
                              circ_class_biomass(ipts,ivm,icir,:,iele)*&
                              circ_class_kill(ipts,ivm,icir,ifm,icut)

                         ! adjust the number of individuals
                         ind(ipts,ivm)=ind(ipts,ivm)-&
                              circ_class_kill(ipts,ivm,icir,ifm,icut)

                         ! Here, it's possible that the number of individuals left
                         ! will be a very, very small amount.  If it's very low,
                         ! we will just move all that biomass to the dead pool
                         ! and set the number of individuals equal to zero,
                         ! effectively killing the circ class.
                         IF(circ_class_n(ipts,ivm,icir) .LE. min_stomate)THEN

                            bm_to_litter(ipts,ivm,:,iele) = bm_to_litter(ipts,ivm,:,iele) + &
                                 circ_class_biomass(ipts,ivm,icir,:,iele)*&
                                 circ_class_n(ipts,ivm,icir)
                            biomass(ipts,ivm,:,iele) = biomass(ipts,ivm,:,iele) - &
                                 circ_class_biomass(ipts,ivm,icir,:,iele)*&
                                 circ_class_n(ipts,ivm,icir)
                            ind(ipts,ivm)=ind(ipts,ivm)-&
                                 circ_class_n(ipts,ivm,icir)

                            ! zero the number of individuals in this circ class
                            circ_class_n(ipts,ivm,icir) = circ_class_n(ipts,ivm,icir) - &
                                 circ_class_n(ipts,ivm,icir)
                            ! 20210503yyt
                       circ_class_cwd(ipts,ivm,icir)=(circ_class_biomass(ipts,ivm,icir,1,iele)+circ_class_biomass(ipts,ivm,icir,2,iele)+circ_class_biomass(ipts,ivm,icir,4,iele)+circ_class_biomass(ipts,ivm,icir,7,iele))*circ_class_n(ipts,ivm,icir)
                        circ_class_leafcwd(ipts,ivm,icir)=circ_class_biomass(ipts,ivm,icir,1,iele)*circ_class_n(ipts,ivm,icir)       
                 write(numout,*) 'circ_class_cwd ', icir, circ_class_cwd(ipts,ivm,icir) 
                        !write(*,*) 'circ_class_cwd ', icir, circ_class_cwd(ipts,ivm,icir)
                          ENDIF

!!$                         IF(test_pft == ivm .AND. test_grid == ipts)&
!!$                              WRITE(numout,*) 'killing after: ',ipts,ivm,icir,ifm,icut,&
!!$                              circ_class_kill(ipts,ivm,icir,ifm,icut),circ_class_n(ipts,ivm,icir)
                     ENDDO ! loop over circ classes

 
                   ELSE

                      !! WARNING !!
                      ! This is not very clean.  We don't use circ classes for grasses 
                      ! and crops.  All mortality is done based on biomass.  However, 
                      ! it's cleaner here to just pass circ_class_kill from lpj_kill, 
                      ! and it is in line with what is done for the forests.  So we take
                      ! the total grass/crop biomass that is scheduled for killing in 
                      ! lpj_kill and define circ_class_kill(:,:,inatural,1) and 
                      ! circ_class_biomass(:,:,1,:,:) so that the product of these two
                      ! matches the amount of biomass scheduled for killing.  We do NOT
                      ! change the number of individuals after killing, either.

                      ! Let us compute the fraction of plants which die of natural
                      ! causes in this step.
                      ! Individuals are not killed for grasses and crops, so we cannot
                      ! calculate the mortality as a function of the individuals

                      ! We renormalize by circ_class_n below.  If it's zero,
                      ! we have an error.  Of course, if it's zero there is no
                      ! reason to be here in the first place.
                      IF(circ_class_n(ipts,ivm,1) .LE. min_stomate)CYCLE

                      ! move the dead biomass to the respective litter pool
                      bm_to_litter(ipts,ivm,:,iele) = bm_to_litter(ipts,ivm,:,iele) + &
                           circ_class_biomass(ipts,ivm,1,:,iele)*&
                           circ_class_kill(ipts,ivm,1,ifm,icut)

                      ! remove the dead biomass
                      biomass(ipts,ivm,:,iele) = biomass(ipts,ivm,:,iele) - &
                           circ_class_biomass(ipts,ivm,1,:,iele)*&
                           circ_class_kill(ipts,ivm,1,ifm,icut)
                      
                      ! circ_class_biomass has to change such that the biomass
                      ! and circ_class_biomass*circ_class_n are in sync.  So we
                      ! will just sync it here.  This has to be done this way
                      ! since circ_class_n never changes for grass/crops.  
                      circ_class_biomass(ipts,ivm,1,:,iele) = biomass(ipts,ivm,:,iele)/&
                           circ_class_n(ipts,ivm,1)

                      ! In order for something to be prescribed, ind must be zero.
                      ! We don't change ind and circ_class_n if we just kill some
                      ! of the biomass, but if we kill all of it we must set
                      ! it to zero in order for it to be prescribed.
                      ! IF(ivm == test_pft) WRITE(numout,*) 'Killing? ',SUM(biomass(ipts,ivm,:,iele))
                      IF(SUM(biomass(ipts,ivm,:,iele)) .LE. min_stomate)THEN
                         circ_class_n(ipts,ivm,:)=zero
                         ind(ipts,ivm)=zero
                      ENDIF
                      

                   ENDIF ! checking for a tree

                   ! This stuff should never been killed again.  
                   ! To make sure of that, reset all the
                   ! variables to zero.
                   ! added by Yitong Yao commented the below line in Jan 05 2020 20:17
                   !circ_class_kill(ipts,ivm,:,ifm,icut)=zero ! to make this var do not equal to 0
                ENDDO
             ENDDO

          ENDDO ! loop over elements

       ENDDO  ! loop over pfts

    END DO ! loop over land points

 !! 4. Check mass balance closure
   
    !  Consider this whole routine as a black box with incoming and outgoing 
    !  fluxes and a change in the mass of the box. Express in absolute 
    !  units gC (or gN), hence, multiply with dt and veget_max. In most routines 
    !  veget_max does not change and could be omitted but a general approach 
    !  was prefered.

    !! 4.1 Calculate pools at the end of the routine
    pool_end = zero
    DO ipar = 1,nparts
       DO iele = 1,nelements
          pool_end(:,:,iele) = pool_end(:,:,iele) + &
               (biomass(:,:,ipar,iele) * veget_max(:,:)) + &
               (bm_to_litter(:,:,ipar,iele) * veget_max(:,:))
       ENDDO
    ENDDO

    !! 4.2 Calculate components of the mass balance
    check_intern(:,:,iatm2land,icarbon) = zero
    check_intern(:,:,iland2atm,icarbon) = -un * zero
    check_intern(:,:,ilat2out,icarbon) = zero
    check_intern(:,:,ilat2in,icarbon) = -un * zero
    check_intern(:,:,ipoolchange,icarbon) = -un * (pool_end(:,:,icarbon) - pool_start(:,:,icarbon))
    closure_intern = zero
    DO imbc = 1,nmbcomp
       closure_intern(:,:,icarbon) = closure_intern(:,:,icarbon) + check_intern(:,:,imbc,icarbon)
    ENDDO
 
    ! Write outcome
    DO ipts=1,npts
       DO ivm=1,nvm
          IF(ABS(closure_intern(ipts,ivm,icarbon)) .LE. min_stomate)THEN
             IF (ld_massbal) WRITE(numout,*) 'Mass balance closure in stomate_kill.f90'
          ELSE
             WRITE(numout,*) 'Error: mass balance is not closed in stomate_kill.f90'
             WRITE(numout,*) '   ipts,ivm; ', ipts,ivm
             WRITE(numout,*) '   Difference is, ', closure_intern(ipts,ivm,icarbon)
             WRITE(numout,*) '   pool_end,pool_start: ', pool_end(ipts,ivm,icarbon), pool_start(ipts,ivm,icarbon)
             IF(ld_stop)THEN
                CALL ipslerr_p (3,'gap_prognostic', 'Mass balance error.','','')
             ENDIF
          ENDIF
       ENDDO
    ENDDO

    IF (bavard.GE.2) WRITE(numout,*) 'Leaving gap_prognostic'

  END SUBROUTINE gap_prognostic


!! ================================================================================================================================
!! SUBROUTINE   : gap_clean
!!
!>\BRIEF        After biomass has been killed, there are some clean-up operations
!!              to do.
!!
!! DESCRIPTION  : If all the biomass has been killed for a PFT, we want to reset
!!                some counters.  There is also the chance that we will have
!!                killed all the biomass in a circ class of trees.  If this is
!!                the case, we want to redistribute the biomass in the remaining
!!                classes so that all circ classes have some biomass in them.
!!                A circ class with no biomass causes allocation to crash.
!!
!! RECENT CHANGE(S): 
!!
!! MAIN OUTPUT VARIABLE(S): ::circ_class_biomass
!!
!! REFERENCE(S)   :
!! - Sitch, S., B. Smith, et al. (2003), Evaluation of ecosystem dynamics,
!!         plant geography and terrestrial carbon cycling in the LPJ dynamic 
!!         global vegetation model, Global Change Biology, 9, 161-185.
!! - Waring, R. H. (1983). "Estimating forest growth and efficiency in relation 
!!         to canopy leaf area." Advances in Ecological Research 13: 327-354.
!!
!! FLOWCHART    : None
!!\n
!_ ================================================================================================================================
 
  SUBROUTINE gap_clean (npts, biomass, circ_class_biomass, &
       circ_class_n, age, everywhere, leaf_age, &
       leaf_frac, senescence, when_growthinit, circ_class_dist, &
       age_stand, PFTpresent, last_cut, mai_count, &
       veget_max, npp_longterm, KF, co2_to_bm, ind, &
       wstress_season, lm_lastyearmax, lignin_struc, lignin_wood, &
       carbon, litter,  vir_wstress_fac, veget, dt_days, &
       forest_managed, bm_to_litter, lab_fac, gdd_from_growthinit,&
       gdd_midwinter, time_hum_min, gdd_m5_dormance, ncd_dormance,&
       moiavail_month, moiavail_week, gpp_week, gpp_daily, resp_maint,&
       resp_growth, npp_daily, use_reserve, mai, pai, ngd_minus5,&
       rue_longterm, previous_wood_volume,&
!!$       t_photo_stress, rue_longterm, previous_wood_volume,&
       tau_eff_leaf, tau_eff_sap, tau_eff_root, moiavail_growingseason,&
       orphan_flux, k_latosa_adapt)

    !! 0. Variable and parameter declaration

    !! 0.1 Input variables 
    INTEGER(i_std), INTENT(in)                       :: npts                   !! Domain size (-)
    REAL(r_std), DIMENSION(ncirc), INTENT(in)        :: circ_class_dist        !! The probability distribution of trees 
                                                                               !! in a circ class in case of a 
                                                                               !! redistribution (unitless).
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: vir_wstress_fac        !! Water stress factor, based on hum_rel_daily
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: veget                  !! Fraction of vegetation type including 
                                                                               !! non-biological fraction (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tau_eff_root           !! Effective root turnover time that accounts
                                                                               !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tau_eff_sap            !! Effective sapwood turnover time that accounts
                                                                               !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tau_eff_leaf           !! Effective leaf turnover time that accounts
                                                                               !! waterstress (days)  
    REAL(r_std), INTENT(in)                          :: dt_days                !! Time step of vegetation dynamics for stomate (days)
    INTEGER(i_std), DIMENSION (:,:), INTENT(in)      :: forest_managed         !! forest management flag
    

    !! 0.2 Output variables

    !! 0.3 Modified variables   
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: biomass                !! Biomass @tex $(gC m^{-2}) $@endtex
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout) :: circ_class_biomass     !! Biomass of the components of the model  
                                                                               !! tree within a circumference
                                                                               !! class @tex $(gC ind^{-1})$ @endtex  
    LOGICAL, DIMENSION(:,:), INTENT(inout)           :: senescence             !! Is the plant senescent? (only for deciduous 
                                                                               !! trees - carbohydrate reserve) (true/false)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: age                    !! Mean age (years)  
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: when_growthinit        !! How many days ago was the beginning of the 
                                                                               !! growing season (days)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: everywhere             !! Is the PFT everywhere in the grid box or very
                                                                               !! localized (after its introduction)
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: leaf_age               !! Leaf age (days)
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: leaf_frac              !! Fraction of leaves in leaf age class 
                                                                               !! (unitless;0-1)
    INTEGER(i_std), DIMENSION(:,:), INTENT(inout)    :: age_stand              !! Age of the forest stand (years)  
    INTEGER(i_std), DIMENSION(:,:), INTENT(inout)    :: last_cut               !! Years since last thinning (years)
    INTEGER(i_std), DIMENSION(:,:), INTENT(inout)    :: mai_count              !! The number of times we've
                                                                               !! calculated the volume increment
                                                                               !! for a stand
    LOGICAL, DIMENSION(:,:), INTENT(inout)           :: PFTpresent             !! PFT present (0 or 1)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: KF                     !! Scaling factor to convert sapwood mass into leaf 
                                                                               !! mass (m)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: k_latosa_adapt         !! Leaf to sapwood area adapted for long 
                                                                               !! term water stress (m)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: veget_max              !! "maximal" coverage fraction of a PFT on the ground
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: co2_to_bm              !! CO2 taken from the atmosphere to get C to create  
                                                                               !! the seedlings @tex (gC.m^{-2}dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: ind                    !! Density of individuals at the stand level 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: wstress_season         !! Water stress factor, based on hum_rel_daily
                                                                               !! (unitless, 0-1) 

    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: npp_longterm           !! "long term" net primary productivity
                                                                               !! @tex ($gC m^{-2} year^{-1}$) @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: lm_lastyearmax         !! last year's maximum leaf mass for each PFT 
                                                                               !! @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: lignin_struc           !! ratio Lignine/Carbon in structural litter,
                                                                               !! above and below ground
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: lignin_wood            !! ratio Lignine/Carbon in woody litter,
                                                                               !! above and below ground
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: carbon                 !! carbon pool: active, slow, or passive 
                                                                               !! @tex ($gC m^{-2}$) @endtex 
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout) :: litter                 !! metabolic and structural litter, above and 
                                                                               !! below ground @tex ($gC m^{-2}$) @endtex 
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: circ_class_n           !! Number of individuals in each circ class
                                                                               !! @tex $(ind m^{-2})$ @endtex  
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: bm_to_litter           !! Transfer of biomass to litter 
                                                                               !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: lab_fac                !! Activity of labile pool factor (??units??)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: gdd_from_growthinit    !! growing degree days, since growthinit 
                                                                               !! for crops
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: gdd_midwinter          !! Growing degree days (K), since midwinter 
                                                                               !! (for phenology) - this is written to the
                                                                               !!  history files 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: time_hum_min           !! Time elapsed since strongest moisture 
                                                                               !! availability (days) 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: gdd_m5_dormance        !! Growing degree days (K), threshold -5 deg 
                                                                               !! C (for phenology)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: ncd_dormance           !! Number of chilling days (days), since 
                                                                               !! leaves were lost (for phenology) 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: moiavail_month         !! "Monthly" moisture availability (0 to 1, 
                                                                               !! unitless) 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: moiavail_week          !! "Weekly" moisture availability 
                                                                               !! (0 to 1, unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: gpp_week               !! Mean weekly gross primary productivity 
                                                                               !! @tex $(gC m^{-2} day^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: gpp_daily              !! Daily gross primary productivity  
                                                                               !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: resp_maint             !! Maintenance respiration  
                                                                               !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: resp_growth            !! Growth respiration  
                                                                               !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: npp_daily              !! Net primary productivity 
                                                                               !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: use_reserve            !! Mass taken from carbohydrate reserve 
                                                                               !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)  :: mai                    !! The mean annual increment
                                                                               !! @tex $(m**3 / m**2 / year)$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)  :: pai                    !! The period annual increment
                                                                               !! @tex $(m**3 / m**2 / year)$ @endtex          
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: ngd_minus5             !! Number of growing days (days), threshold 
                                                                               !! -5 deg C (for phenology)   
!!$    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: t_photo_stress         !! Temperature stress for CEXCHANGE 
!!$                                                                               !! photosynthesis (0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: rue_longterm           !! Longterm radiation use efficiency 
                                                                               !! (??units??) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)  :: previous_wood_volume   !! The volume of the tree trunks
                                                                               !! in a stand for the previous year.
                                                                               !! @tex $(m**3 / m**2 )$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: moiavail_growingseason !! Mean growingseason moisture 
                                                                               !! availability (0 to 1, unitless)
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: orphan_flux            !! Storage for fluxes of PFTs that no longer exist. 
                                                                               !! Following the total destruction of a PFT in LCC 
                                                                               !! (veget_max_new = 0), a flux from before LCC needs a 
                                                                               !! storage @tex $(gC m^{-2} dtslow^{-1})$ @endtex

    !! 0.4 Local variables
    REAL(r_std)                                      :: share_young            !! Share of the veget_max of the youngest age class
                                                                               !! (unitless, 0-1)
    INTEGER(i_std)                                   :: ipts,ivm,ilage         !! Indices
    INTEGER(i_std)                                   :: icir,jcir,ilev         !! Indices
    INTEGER(i_std)                                   :: ipar,iele,imbc         !! Indices
    INTEGER(i_std)                                   :: ilit,icarb             !! Indices
    INTEGER(i_std)                                   :: inc                    !! Indices
    LOGICAL                                          :: lredistribute          !! Flag if we need to redistribute individuals
                                                                               !! among the circ classes
    INTEGER,DIMENSION(nvm)                           :: nkilled                !! the number of grid points at which a given
                                                                               !! given PFT's biomass has been reduced to 
                                                                               !! zero
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements)&
                                                     :: check_intern           !! Contains the components of the internal
                                                                               !! mass balance chech for this routine
                                                                               !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: closure_intern         !! Check closure of internal mass balance
                                                                               !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: pool_start             !! Start pool of this routine 
                                                                               !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: pool_end               !! End pool of this routine 
                                                                               !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: old_trees_left
    REAL(r_std), DIMENSION(ncirc)                    :: circ_class_n_new
    REAL(r_std), DIMENSION(ncirc,nparts)             :: circ_class_biomass_new
    REAL(r_std)                                      :: trees_needed
    LOGICAL                                          :: lyoungest
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements):: new_biomass            !! Temporary variable for biomass 
                                                                               !! @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nleafages)       :: new_leaf_frac          !! Temporary variable for leaf_frac (unitless;0-1)
    REAL(r_std), DIMENSION(npts,nvm)                 :: new_ind                !! Temporary variable for ind @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: est_circ_class_n       !! Temporary variable for circ_class_n of the 
                                                                               !! established vegetation @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: tmp_circ_class_n       !! Temporary variable for circ_class_n of the 
                                                                               !! established vegetation @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)           :: new_circ_class_n       !! Temporary variable for circ_class_n of the
                                                                               !! new vegetation
                                                                               !! @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc,nparts,nelements)   :: est_circ_class_biomass !! Temporary variable for circ_class_biomass of the
                                                                               !! established vegetation @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc,nparts,nelements)   :: tmp_circ_class_biomass !! Temporary variable for circ_class_biomass of the
                                                                               !! established vegetation @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements) &
                                                     :: new_circ_class_biomass !! Temporary variable for circ_class_biomass of the
                                                                               !! new vegetation @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: est_circ_class_dia     !! Variable to store temporary values for 
                                                                               !! circ_class (m)
    REAL(r_std), DIMENSION(ncirc)                    :: tmp_circ_class_dia     !! Variable to store temporary values for 
                                                                               !! circ_class (m)
    REAL(r_std), DIMENSION(npts,nvm,ncirc)           :: new_circ               !! Temporary variable for circ (m)
    REAL(r_std), DIMENSION(npts,nvm)                 :: new_co2_to_bm          !! Temporary variable for co2_to_bm 
                                                                               !! @tex (gC.m^{-2}dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                 :: new_age                !! Temporary variable for age (years)
    REAL(r_std), DIMENSION(npts,nvm)                 :: new_lm_lastyearmax     !! Temporary variable for lm_last_year 
                                                                               !! @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(npts,nvm)                 :: new_everywhere         !! Temporary variable for everywhere (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm)                 :: dum_when_growthinit    !! Dummy for when_growthinit (days)
    REAL(r_std), DIMENSION(npts,nvm)                 :: dum_npp_longterm       !! Dummy for npp_long_term
                                                                               !! @tex ($gC m^{-2} year^{-1}$) @endtex
    INTEGER(i_std) :: ivma
    LOGICAL, DIMENSION(npts,nvm)                     :: dum_PFTpresent         !! Dummy for PFTpresent (0 or 1)
    LOGICAL, DIMENSION(npts,nvm)                     :: dum_senescence         !! Dummy for senescence (true/false)
    REAL(r_std)                                      :: moi_bank               !! bank to store moiavail_growingseason of the cleared
                                                                               !! PFT's (unitless; 0-1)
    REAL(r_std), DIMENSION(npts,nvm)                 :: loss_gain              !! The same as delta_veget but distributed of all
                                                                               !! age classes and thus taking the age-classes into 
                                                                               !! account (unitless, 0-1)
    REAL(r_std)                                      :: total_losses           !! Sum of losses only in delta_veget (unitless, 0-1)
    REAL(r_std), DIMENSION(nlitt,nlevs,nelements)    :: litter_bank            !! Bank to store the litter that becomes available when
                                                                               !! part of a PFT is removed @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(ncarb)                    :: soil_bank              !! Bank to store soil carbon that becomes available when
                                                                               !! part of a PFT is removed  @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(nlevs)                    :: struct_ltr_bank        !! Bank to store the lignin in structural litter that 
                                                                               !! becomes available when part of a PFT is removed 
                                                                               !! (0-1,unitless)
    REAL(r_std),DIMENSION(nlevs)                     :: woody_ltr_bank         !! Bank to store the lignin in woody litter that 
                                                                               !! becomes available when part of a PFT is removed 
                                                                               !! (0-1,unitless)
    REAL(r_std),DIMENSION(nvm,nparts,nelements)      :: fresh_litter           !! Pool of fresh litter that is being released during 
                                                                               !! LCC @tex ($gC m^{-2}$) @endtex 
    REAL(r_std),DIMENSION(nparts,nelements)          :: fresh_ltr_bank         !! Bank to store all the fresh litter from site 
                                                                               !! clearing during LCC. Weighted value of fresh_litter
                                                                               !! @tex ($gC m^{-2}$) @endtex
    INTEGER(i_std) :: iyoung
    REAL(r_std), DIMENSION(npts,nvm,norphans,nelements) &
                                               :: orphan_flux_local            !! Storage for fluxes of PFTs that no longer exist. 
                                                                               !! This is only for co2bm at the moment, since that
                                                                               !! is the only one used in this routine. Following the 
                                                                               !! total destruction of a PFT by death
                                                                               !! (veget_max_new = 0), a flux from before death needs  
                                                                               !! storage @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(nlitt,nlevs)                :: litter_weight_young  !! The fraction of litter on the young
                                                                               !! PFT.
                                                                               !! @tex $-$ @endtex
    REAL(r_std), DIMENSION(nparts)                     :: v1                   !! temporay variable for sorting circ_class_biomass
    REAL(r_std)                                        :: v2                   !! temporay variables for sorting circ_class_n
    REAL(r_std)                                        :: sort                 !! flag triggering sorting routine (0 or 1)
    REAL(r_std), DIMENSION(ncirc)                      :: tree_size            !! temporay variables for checking the biomass
                                                                               !! for each circ class @tex ($gC tree^{-1}$) @endtex
    
!_ ================================================================================================================================

    IF (bavard.GE.2) WRITE(numout,*) 'Entering gap_clean.'

 !! 1. Initialize check for mass balance closure
  
    !  The mass balance is calculated at the end of this routine
    !  in section .
    pool_start = zero
    DO iele = 1,nelements
       DO ipar = 1,nparts
          !  Initial biomass
          pool_start(:,:,iele) = pool_start(:,:,iele) + &
               (biomass(:,:,ipar,iele) * veget_max(:,:)) 
       ENDDO

       ! The litter and soil carbon pools can be moved around
       ! if age classes are changed.
       
       ! Litter pool (gC m-2) *  (m2 m-2) 
       DO ilit = 1,nlitt
          DO ilev = 1,nlevs
             pool_start(:,:,iele) = pool_start(:,:,iele) + &
                  litter(:,ilit,:,ilev,iele) * veget_max(:,:)
          ENDDO
       ENDDO

       ! Soil carbon (gC m-2) *  (m2 m-2)
       DO icarb = 1,ncarb
          pool_start(:,:,iele) = pool_start(:,:,iele) + &
               carbon(:,icarb,:) * veget_max(:,:)
       ENDDO

    ENDDO
    ! It is possible that the orphan fluxes are not zero here.  For example,
    ! if we prescribed biomass at the beginning of this timestep,
    ! ico2bm will be some value.  We are only interested in what happens in this
    ! step for the moment, so we use a local variable for this flux.
    orphan_flux_local(:,:,:,:)=zero

 !! 2. Redistribute biomass
  
    DO  ipts = 1,npts ! loop over land_points

       DO ivm = 2,nvm  ! loop over #PFT


          IF(veget_max(ipts,ivm) == zero)THEN
              ! this vegetation type is not present, so no reason to do the 
              ! calculation
               CYCLE
          ENDIF



          ! First we check to see if one of the circ classes is empty.  We only need
          ! to do this if there is some biomass, since we don't care if all
          ! of the circ classes are empty.
          IF(SUM(biomass(ipts,ivm,:,icarbon)) .GT. min_stomate)THEN
             IF(is_tree(ivm))THEN

                !+++++ CHECK +++++
                ! Try redistributing at every timestep
!!$                lredistribute=.FALSE.
!!$                DO icir=1,ncirc
!!$                   IF(SUM(circ_class_biomass(ipts,ivm,icir,:,icarbon))*&
!!$                        circ_class_n(ipts,ivm,icir) .LT. min_stomate) &
!!$                        lredistribute=.TRUE.
!!$                ENDDO

                lredistribute=.TRUE.
                !+++++++++++++++++

                IF(lredistribute) THEN

                   IF(ld_biomass .AND. ivm==test_pft)THEN
                      WRITE(numout,*) 'Start of redistributing biomass in gap_clean'
                      WRITE(numout,*) 'ipts,ivm',ipts,ivm
                      WRITE(numout,*) 'circ_class_n, ',circ_class_n(ipts,ivm,:)
                      WRITE(numout,*) 'circ_class_biomass, ', &
                           SUM(circ_class_biomass(ipts,ivm,:,:,icarbon),2)
                   ENDIF

                   ! What we want to do is recreate the circumference class distribution,
                   ! since if we have hit this point we have at least one class that
                   ! is empty and this will cause the allocation to crash.  

                   ! The first step is to decide of the distribution of individuals 
                   ! that we want in each class, for example, a uniform or exponential 
                   ! distribution.
                   ! Make this an input parameter.

                   ! Now we need to populate the new distribution of trees among
                   ! the circumference classes, and move the heartwood and sap masses 
                   ! to the new distributions. We also need to move the other pools.  
                   ! This is tricky because the allometric relations for the new
                   ! tree sizes will NOT give the same amount of biomass for
                   ! the non-woody pools.  Let us first try it just distributing
                   ! everything equally, and then if that doesn't work we can
                   ! redistribute the non-woody pools more cleverly, even if that 
                   ! means we change the total amount of biomass we have in this
                   ! stand.  I want to try it this way because we conserve biomass 
                   ! this way, and I hope that the stress on the trees will be
                   ! small enough that it doesn't cause problems.  This
                   ! redistribution should only happen rarely, so the trees
                   ! should have a chance to equilibrate.

                   circ_class_n_new(:)=circ_class_dist(:)*SUM(circ_class_n(ipts,ivm,:))
                   circ_class_biomass_new(:,:)=zero

                   ! now we populate the new classes
                   old_trees_left(:)=circ_class_n(ipts,ivm,:)

                   ! The subsequent calculation nicely closes the carbon
                   ! balance but within the precision 10-16. If there are
                   ! enough trees (first case in the IF-loop), we introduce
                   ! a precision error in the number of trees. This implies
                   ! that when the number of trees is very small, the
                   ! precision issue can become a numerical issue. This is 
                   ! especially true for the largest circumference classes 
                   ! because they contains the fewest individuals. 
                   ! We observed the largest circumference classes 
                   ! containing 10-4 to 10-5 gC less than the previous 
                   ! class. The rest of the code shouldn't care too much 
                   ! whether the circumferences classes are in order or not 
                   ! but to reduce the chances this problem occurs
                   ! we inverted the DO-loops. That way the precision error
                   ! occurs in carried to the smaller diameter classes 
                   ! which have more individuals and so the precision error
                   ! stays a precision issue. Note that this routine is
                   ! called every day so the problem is likely to be 
                   ! corrected the next day.
                   DO icir=ncirc,1,-1

                      trees_needed=circ_class_n_new(icir)

                      DO jcir=ncirc,1,-1

                         IF(trees_needed .LE. zero)EXIT

                         !+++TEMP+++
                         IF(ld_biomass .AND. ivm==test_pft)THEN
                            WRITE(numout,*) 'start of loop'
                            WRITE(numout,*) 'circ_class_biomass_new, ',&
                                 icir,SUM(circ_class_biomass_new(icir,:))
                            WRITE(numout,*) 'old_trees_left, ', &
                                 jcir, old_trees_left(jcir)
                            WRITE(numout,*) 'trees needed, ',trees_needed
                         ENDIF
                         !++++++++++

                         IF(old_trees_left(jcir) .GE. trees_needed )THEN

                            ! we can get all the trees we need from this class
                            ! and don't have to continue searching
                            circ_class_biomass_new(icir,:)=circ_class_biomass_new(icir,:)+&
                                 circ_class_biomass(ipts,ivm,jcir,:,icarbon)*trees_needed
                            old_trees_left(jcir)=old_trees_left(jcir)-trees_needed
                            trees_needed = zero

                            !+++TEMP+++
                            IF(ld_biomass .AND. ivm==test_pft)THEN
                               WRITE(numout,*) 'enough trees'
                            ENDIF
                            !++++++++++
   
                            EXIT

                         ELSE

                            ! the trees in this class are not sufficient, so we
                            ! need to move all of them to the new class.
                            circ_class_biomass_new(icir,:)=circ_class_biomass_new(icir,:)+&
                                 circ_class_biomass(ipts,ivm,jcir,:,icarbon)*&
                                 old_trees_left(jcir)
                            trees_needed = trees_needed-old_trees_left(jcir)
                            old_trees_left(jcir)=zero

                            !+++TEMP+++
                            IF(ld_biomass .AND. ivm==test_pft)THEN
                               WRITE(numout,*) 'not enough trees'
                            ENDIF
                            !++++++++++

                         ENDIF
                         
                         !+++TEMP+++
                         IF(ld_biomass .AND. ivm==test_pft)THEN
                            WRITE(numout,*) 'start of loop'
                            WRITE(numout,*) 'circ_class_biomass_new, ',&
                                 icir,SUM(circ_class_biomass_new(icir,:))
                            WRITE(numout,*) 'old_trees_left, ', &
                                 jcir, old_trees_left(jcir)
                            WRITE(numout,*) 'trees needed, ',trees_needed
                         ENDIF
                         !++++++++++

                      ENDDO ! jcir=1,ncirc

                   ENDDO ! icir=1,ncirc

                   ! right now, circ_class_biomass_new gives the total biomass
                   ! in each class, but it should only be for a model tree. 
                   ! So let's normalize it.
                   DO icir=1,ncirc
                      circ_class_biomass_new(icir,:)=circ_class_biomass_new(icir,:)/&
                           circ_class_n_new(icir)
                   ENDDO

                   ! Check whether the order was preserved. We expect that the
                   ! biomass of an individual tree increases with an increasing
                   ! circ class. This is probably not important for the correct
                   ! functioning of the model. We try to preserve the order as
                   ! an additional mean to control the quality of the simulations
                   ! If the order was not preserved we will sort the biomasses.
                   tree_size(:)=zero
                   DO icir=1,ncirc

                      tree_size(icir)=SUM(circ_class_biomass_new(icir,:))
                      
                   ENDDO
                   sort = zero
                   DO icir=2,ncirc

                      IF(tree_size(icir) .LT. tree_size(icir-1))THEN
                         
                         ! The order is not preserved
                         sort = un
                         EXIT

                      ENDIF

                   ENDDO
                   
                   ! Sort the biomasses. Note that at the same time we also 
                   ! have to sort circ_class_n. We made use of the Shell sort
                   ! method
                   IF (sort==1) THEN

                      !+++TEMP+++
                      IF(ld_biomass)THEN
                         WRITE(numout,*) 'gap_clean: sort begin', &
                              SUM(circ_class_biomass_new(:,:),2),&
                              circ_class_n_new(:)
                      ENDIF
                      !++++++++++
                      
                      inc = 1
                      DO
                         inc=3*inc+1
                         IF (inc .GT. ncirc) EXIT
                      ENDDO
                      DO
                         inc = inc/3
                         DO icir=inc+1,ncirc
                            v1(:) = circ_class_biomass_new(icir,:)
                            v2 = circ_class_n_new(icir)
                            jcir=icir
                            DO
                               IF (SUM(circ_class_biomass_new(jcir-inc,:)) .LE. SUM(v1(:))) EXIT
                               circ_class_biomass_new(jcir,:) = &
                                    circ_class_biomass_new(jcir-inc,:)
                               circ_class_n_new(jcir) = circ_class_n_new(jcir-inc) 
                               jcir = jcir-inc
                               IF (jcir .LE. inc) EXIT
                            ENDDO
                            circ_class_biomass_new(jcir,:)=v1(:)
                            circ_class_n_new(jcir)=v2
                         ENDDO
                         IF (inc .LE. 1) EXIT
                      ENDDO

                      !+++TEMP+++
                      IF(ld_biomass)THEN
                         WRITE(numout,*) 'gap_clean: sort end', &
                              SUM(circ_class_biomass_new(:,:),2),&
                              circ_class_n_new(:)
                      ENDIF
                      !++++++++++

                   ENDIF

                   ! now update the variables that pass this information around
                   DO icir=1,ncirc
                      circ_class_biomass(ipts,ivm,icir,:,icarbon)=circ_class_biomass_new(icir,:)
                      circ_class_n(ipts,ivm,icir)=circ_class_n_new(icir)
                   ENDDO


                ENDIF ! redistribute
                
                !---TEMP---
                IF(ld_biomass .AND. ivm==test_pft)THEN
                   WRITE(numout,*) 'End of redistributing biomass in gap_clean'
                   WRITE(numout,*) 'ipts,ivm',ipts,ivm
                   WRITE(numout,*) 'circ_class_n, ',circ_class_n(ipts,ivm,:)
                   WRITE(numout,*) 'circ_class_biomass, ', &
                        SUM(circ_class_biomass(ipts,ivm,:,:,icarbon),2)
                ENDIF
                !----------

             ELSE

                ! we have biomass here and it's not a tree.  Don't need to do anything.

             ENDIF

          ELSE 

             ! all the biomass was killed for this site.  Some flags to
             ! be reset in this case.

             IF(PFTpresent(ipts,ivm))THEN
                
                nkilled(ivm)=nkilled(ivm)+1 ! purely an informational variable

                !! 1.5 Reinitialize vegetation characteristics in STOMATE
                senescence(ipts,ivm) = .FALSE.
                age(ipts,ivm) = zero
                when_growthinit(ipts,ivm) = large_value 

                !! 1.6 Update leaf ages
                DO ilage = 1, nleafages
                   
                   leaf_age(ipts,ivm,ilage) = zero 
                   leaf_frac(ipts,ivm,ilage) = zero 
                   
                ENDDO

                ! This used to be done in lpj_kill.  If we have grasses,
                ! we reset the long term GPP.  This value as 500 in the
                ! old code.  We externalized the variable, but we are
                ! leaving the default at 500.  Many PFTs should have
                ! a long term value of less than 500, so someone could
                ! do some work on this in the future.
                IF (.NOT. is_tree(ivm) .AND. .NOT. control%ok_constant_mortality) THEN
                   
                   npp_longterm(ipts,ivm) = npp_reset_value(ivm)

                ENDIF

                
             ENDIF ! is PFT present?

          ENDIF ! check if biomass is equal to zero

          ! If we have no vegetation left, this stand has been completely cleared.
          ! We have to reset the age, regardless if it was for natural or human reasons.
          IF(is_tree(ivm))THEN

             IF( SUM(biomass(ipts,ivm,:,icarbon)) .LT. min_stomate)THEN

                age_stand(ipts,ivm)=0

                IF(ld_forestry .OR. ld_kill)THEN
                   WRITE(numout,*) 'Getting reset in stomate_kill'
                   WRITE(numout,*) 'ivm,ipts',ivm,ipts
                ENDIF
                
                ! We should reset these flags as well, since we are
                ! doing management somewhere.
                last_cut(ipts,ivm)=0
                mai_count(ipts,ivm)=0

                ! If we are using age classes, we need to reset a lot of counters
                ! and change veget_max, moving veget_max from this age class to
                ! the lowest age class.  If veget_max is greater than zero and there
                ! is no biomass, prescribe will attempt to grow trees here in the
                ! next timestep.  This is fine if we have no age classes, but it doesn't
                ! make any sense to prescribe trees that are 50 years old.  We should
                ! only prescribe trees which are 0 years old.
                !  This is essentially the same code that is found in land cover change,
                ! since the same simulation happens there.  Notice that here we do not
                ! need to use the _bank variables, since we assume that if a stand dies
                ! due to natural causes it will always be replaced by the same species.
                ! We also assume that if a stand is clearcut, the model will replace it
                ! by the same species.  To do so otherwise would require something more
                ! like a DGVM.
                IF(nagec .GT. 1)THEN

                   ! First, we need to find the youngest age class of this PFT.
                   iyoung=start_index(agec_group(ivm))
                   IF(ivm == iyoung)THEN
                      lyoungest=.TRUE.
                   ELSE
                      lyoungest=.FALSE.
                   ENDIF

                   IF(lyoungest)THEN

                      ! We don't need to do anything.  Things will be handled properly with
                      ! prescribe in the next step.

                   ELSE

                      ! All of our counters for this stand are zero.  We need to merge that
                      ! with the youngest age class.  There are a couple possibilities.
                      ! One is that nothing exists right now in the youngest age class, in 
                      ! which case the veget_max of the old age class becomes that of the 
                      ! youngest and we prescribe.  Another possibility is that something 
                      ! does exist in the youngest age class, in which case we prescribe 
                      ! to the current age class and then merge biomass with the youngest.  
                      ! A third is that both the youngest age class and the current age 
                      ! class have died.  In that case we can do the same thing as if there 
                      ! is no vegetation in the small age class, we just have to make sure 
                      ! to add the veget_max and not replace it.
                      ! Veget_max after PFT expansion. As ::veget_max is used in the 
                      ! IF-statements we won't update it until the end of this routine. If 
                      ! we would updatebefore that, the same PFT may be subject to 
                      ! conflicting actions.
                      share_young = veget_max(ipts,iyoung) / &
                           ( veget_max(ipts,iyoung) + veget_max(ipts,ivm) )

                      ! We also need a scaling factor which includes the litter
                      DO ilev=1,nlevs
                         DO ilit=1,nlitt
                            IF(litter(ipts,ilit,iyoung,ilev,icarbon) .GT. min_stomate)THEN

                               litter_weight_young(ilit,ilev)=&
                                    litter(ipts,ilit,iyoung,ilev,icarbon)*veget_max(ipts,iyoung)/ &
                                    (litter(ipts,ilit,ivm,ilev,icarbon)*veget_max(ipts,ivm) + &
                                    litter(ipts,ilit,iyoung,ilev,icarbon)*veget_max(ipts,iyoung))

                            ELSE

                               litter_weight_young(ilit,ilev)=zero

                            ENDIF
                         END DO
                      ENDDO

                      IF( SUM(biomass(ipts,iyoung,:,icarbon)) .LT. min_stomate)THEN

                         IF(ld_agec)THEN
                            WRITE(numout,*) 'Merging our biomass to the youngest age class.'
                            WRITE(numout,*) 'No current biomass in the youngest age class.'
                            WRITE(numout,*) 'ipts,iyoung,ivm: ',ipts,iyoung,ivm
                            WRITE(numout,*) 'share_young: ',share_young
                            WRITE(numout,*) 'veget_max(young and current): ',&
                                 veget_max(ipts,iyoung),veget_max(ipts,ivm)
                         ENDIF

                         ! Is there anything in the smallest age class?  It
                         ! does not matter if the youngest age class died in this
                         ! step or not.                         
                         veget_max(ipts,iyoung)=veget_max(ipts,iyoung)+veget_max(ipts,ivm)
                         
                         moiavail_growingseason(ipts,iyoung) = &
                              share_young * moiavail_growingseason(ipts,iyoung) + &
                              moiavail_growingseason(ipts,ivm) * &
                              (un - share_young)

                         ! I am not sure why we merge wstress here.  wstress is recalculated
                         ! everyday from moiavail_growingseason.
                         wstress_season(ipts,iyoung) = &
                              share_young * wstress_season(ipts,iyoung) + &
                              wstress_season(ipts,ivm) * (un - share_young)

                         ! The litter variables also need to be merged, since these will not
                         ! get updated in prescribe in the next step.
                         litter(ipts,:,iyoung,:,:) =   &
                              share_young * litter(ipts,:,iyoung,:,:) + &
                              litter(ipts,:,ivm,:,:) * &
                              (un - share_young)
                         carbon(ipts,:,iyoung) =  &
                              share_young *  carbon(ipts,:,iyoung) + &
                               carbon(ipts,:,ivm) * &
                              (un - share_young)
                         DO ilev=1,nlevs
                            lignin_struc(ipts,iyoung,ilev) = &
                                 litter_weight_young(istructural,ilev) * &
                                 lignin_struc(ipts,iyoung,ilev) + &
                                 (un - litter_weight_young(istructural,ilev)) * &
                                 lignin_struc(ipts,ivm,ilev) 
                            lignin_wood(ipts,iyoung,ilev) = &
                                 litter_weight_young(iwoody,ilev) * &
                                 lignin_wood(ipts,iyoung,ilev) + &
                                 (un - litter_weight_young(iwoody,ilev) ) * &
                                 lignin_wood(ipts,ivm,ilev) 
                         ENDDO

                         bm_to_litter(ipts,iyoung,:,:) = &
                              share_young * bm_to_litter(ipts,iyoung,:,:) + &
                              bm_to_litter(ipts,ivm,:,:) * &
                              (un - share_young)


                      ELSE

                         ! There are two important things to do here.  First,
                         ! we need to prescribe biomass to this PFT, since it
                         ! is currently empty.  Then we need to merge this biomass
                         ! with what is already in the youngest age class of this PFT.

                         ! We want to make use of prescribe_prognostic but it should 
                         ! be noted that the youngest PFT already contains biomass. 
                         ! Therefore we will create a temporary PFT without biomass which 
                         ! can be used to establish the  new vegetation within the 
                         ! youngest. For most of the INTENT(inout)  variables of 
                         ! prescribe_prognostic we receive temporary variables back this 
                         ! helps us to calculate the weighted mean of the newly established 
                         ! vegetation and the vegetation that is already there. For some 
                         ! other variables we receive dummies as we are simply not
                         ! going to use these values but will continue using the values of the
                         ! vegetation that is already there.

                         IF(ld_agec)THEN
                            WRITE(numout,*) 'Merging our biomass to the youngest age class.'
                            WRITE(numout,*) 'Biomass already in the youngest age class.'
                            WRITE(numout,*) 'ipts,iyoung,ivm: ',ipts,iyoung,ivm
                            WRITE(numout,*) 'share_young: ',share_young
                            WRITE(numout,*) 'veget_max(young and current): ',&
                                 veget_max(ipts,iyoung),veget_max(ipts,ivm)
                         ENDIF

                         !! 5.2.3.1 Initialize new and dummy variables
                         new_biomass(:,:,:,:) = zero
                         new_circ_class_biomass(:,:,:,:,:) = zero
                         new_ind(:,:) = zero
                         new_circ_class_n(:,:,:) = zero
                         new_lm_lastyearmax(:,:) = zero
                         new_age(:,:) = zero
                         new_leaf_frac(:,:,:) = zero
                         new_co2_to_bm(:,:) = zero
                         new_everywhere(:,:) = zero 
                         dum_when_growthinit(:,:) = large_value
                         dum_npp_longterm(:,:) = zero
                         dum_PFTpresent(:,:) = .TRUE.
                         dum_senescence(:,:) = .FALSE.
                
                         !! 5.2.3.2 Merge the new and already available vegetation

                         ! Assign value to moiavail_growingseason
                         ! We do this differently from LCC, because we are replanting
                         ! the same stand that we just cut.
                         moiavail_growingseason(ipts,iyoung) = &
                              moiavail_growingseason(ipts,iyoung) * &
                              share_young + moiavail_growingseason(ipts,ivm) * (un - share_young)
                         wstress_season(ipts,iyoung) = wstress_season(ipts,iyoung) * &
                              share_young + wstress_season(ipts,ivm) * (un - share_young)
     
                         !! 5.2.3.3 Initialize the newly established vegetation: 
                         !  Initialize the newly established vegetation: density of 
                         !  individuals, biomass, allocation factors, leaf age distribution, 
                         !  etc to some reasonable value ::veget_max is not updated yet, 
                         !  therefore loss_gain is passed in the argument list.
                         !  For loss_gain, we only want to replant the current PFT.
                         loss_gain(:,:)=zero
                         loss_gain(ipts,ivm)=veget_max(ipts,ivm)
                         CALL prescribe_prognostic (npts,loss_gain,veget, dt_days,dum_PFTpresent, &
                              new_everywhere, dum_when_growthinit, new_biomass, new_leaf_frac, &
                              new_ind, new_circ_class_n, new_circ_class_biomass, new_co2_to_bm, &
                              forest_managed, KF, &
                              dum_senescence, new_age, dum_npp_longterm, new_lm_lastyearmax, &
                              tau_eff_leaf, tau_eff_sap, tau_eff_root, k_latosa_adapt)

                         !! 5.2.3.3.1 Merge biomass of new and already available trees
                         ! Unlike LCC, we are only at this point if we are dealing
                         ! with forests, so we don't have to check for that here.

                         ! Copy the number of individuals and the biomass of the established 
                         ! vegetation to a temporary variable. In the subroutine ::circ_class_n
                         ! and ::circ_class_biomass will be overwritten with the characteristics
                         ! of the merged vegetation
                         ! The term "established" here refers to the youngest age class, while
                         ! "tmp" is the prescribed vegetation that we just created.
                         est_circ_class_n(:) = circ_class_n(ipts,iyoung,:)
                         est_circ_class_biomass(:,:,:) = circ_class_biomass(ipts,iyoung,:,:,:)
                         tmp_circ_class_n(:) = new_circ_class_n(ipts,ivm,:)
                         tmp_circ_class_biomass(:,:,:) = new_circ_class_biomass(ipts,ivm,:,:,:)
                         
                         ! Store circumference (m) in a temporary variable that can be 
                         ! sorted from small to large - note that the dimension of these
                         ! variables are 2*ncirc
                         est_circ_class_dia(:) = wood_to_dia(est_circ_class_biomass(:,:,icarbon),ivm)
                         tmp_circ_class_dia(:) = wood_to_dia(tmp_circ_class_biomass(:,:,icarbon),ivm)

                         ! Merge the biomass 
                         CALL merge_biomass_pfts(npts, share_young, circ_class_n, &
                              est_circ_class_n, tmp_circ_class_n, circ_class_biomass, &
                              est_circ_class_biomass, &
                              tmp_circ_class_biomass, ind, biomass, circ_class_dist, &
                              ipts, iyoung, est_circ_class_dia, tmp_circ_class_dia)

                         !! 5.2.3.4 Calculate the PFT characteristics of the merged PFT
                         !  Take the weighted mean of the existing vegetation and the new 
                         !  vegetation in this PFT. Note that co2_to_bm is in gC. m-2 dt-1 
                         !  so we should also take the weighted mean (rather than sum if
                         !  this where absolute values).
                         lm_lastyearmax(ipts,iyoung) = share_young * lm_lastyearmax(ipts,iyoung) + &
                              (un - share_young) * new_lm_lastyearmax(ipts,ivm)
                         age(ipts,iyoung) = share_young * age(ipts,iyoung) + &
                              (un - share_young) * new_age(ipts,ivm)
                         leaf_frac(ipts,iyoung,:) = share_young * leaf_frac(ipts,iyoung,:) + &
                              (un - share_young) * new_leaf_frac(ipts,ivm,:)
                         co2_to_bm(ipts,iyoung) = share_young * co2_to_bm(ipts,iyoung) + &
                              (un - share_young) * new_co2_to_bm(ipts,ivm)

                         ! Update the orphan flux for the CO2 turned into biomass by prescribe,
                         ! as well as the area of vegetation regrown.
                         orphan_flux_local(ipts,ivm,ico2bm,icarbon) = new_co2_to_bm(ipts,ivm)
                         orphan_flux_local(ipts,ivm,ivegnew,icarbon) = veget_max(ipts,ivm)

                         ! Everywhere deals with the migration of vegetation. Copy the
                         ! status of the most migrated vegetation for the whole PFT
                         everywhere(ipts,iyoung) = MAX(everywhere(ipts,iyoung), &
                              new_everywhere(ipts,ivm))

                         ! The new soil&litter pools are the weighted mean of the newly 
                         ! established vegetation for that PFT and the vegetation that 
                         ! already exists in that PFT.  Notice that we do not use the
                         ! "bank" concept present in LCC because we are replanting
                         ! the same PFT which already existed, just in a younger class.
                         litter(ipts,:,iyoung,:,:) = share_young * litter(ipts,:,iyoung,:,:) + &
                              (un - share_young) * litter(ipts,:,ivm,:,:)
                         carbon(ipts,:,iyoung) =  share_young * carbon(ipts,:,iyoung) + &
                              (un - share_young) * carbon(ipts,:,ivm)
                         DO ilev=1,nlevs
                            lignin_struc(ipts,iyoung,ilev) = &
                                 litter_weight_young(istructural,ilev) * &
                                 lignin_struc(ipts,iyoung,ilev) + &
                                 (un - litter_weight_young(istructural,ilev)) * &
                                 lignin_struc(ipts,ivm,ilev) 
                            lignin_wood(ipts,iyoung,ilev) = &
                                 litter_weight_young(iwoody,ilev) * &
                                 lignin_wood(ipts,iyoung,ilev) + &
                                 (un - litter_weight_young(iwoody,ilev) ) * &
                                 lignin_wood(ipts,ivm,ilev) 
                         ENDDO
                         bm_to_litter(ipts,iyoung,:,:) = share_young * &
                              bm_to_litter(ipts,iyoung,:,:) + & 
                              (un - share_young) * bm_to_litter(ipts,ivm,:,:)

                         ! Now move the veget_max from the current class to the young one.
                         veget_max(ipts,iyoung)=veget_max(ipts,iyoung)+veget_max(ipts,ivm)

                      ENDIF
                
                      ! Reset all these variables, since the PFT is dead.
                      PFTpresent(ipts,ivm) = .FALSE.
                      veget_max(ipts,ivm) = zero
                      lm_lastyearmax(ipts,ivm) = zero
                      age(ipts,ivm) = zero
                      leaf_frac(ipts,ivm,:) = zero
                      co2_to_bm(ipts,ivm) = zero
                      everywhere(ipts,ivm) = zero
                      litter(ipts,:,ivm,:,:) = zero
                      carbon(ipts,:,ivm) = zero
                      lignin_struc(ipts,ivm,:) = zero
                      lignin_wood(ipts,ivm,:) = zero
                      bm_to_litter(ipts,ivm,:,:) = zero
                      biomass(ipts,ivm,:,:) = zero
                      ind(ipts,ivm) = zero
                      
                      ! Keep ::biomass and circ_class_biomass synchronized
                      circ_class_biomass(ipts,ivm,:,:,:) = zero
                      circ_class_n(ipts,ivm,:) = zero
                      
                   ENDIF ! If this is the youngest age class

                ENDIF ! If we are using age classes

             ENDIF ! If we've killed all our biomass in ivm

          ENDIF ! is_tree

       ENDDO  ! loop over pfts

    END DO ! loop over land points

  !! 3. Check mass balance closure
   
    !  Consider this whole routine as a black box with incoming and outgoing 
    !  fluxes and a change in the mass of the box. Express in absolute 
    !  units gC (or gN), hence, multiply with dt and veget_max. In most routines 
    !  veget_max does not change and could be omitted but a general approach 
    !  was prefered.

    !! 3.1 Calculate pools at the end of the routine
    pool_end = zero
    DO iele = 1,nelements
       DO ipar = 1,nparts
          pool_end(:,:,iele) = pool_end(:,:,iele) + &
               (biomass(:,:,ipar,iele) * veget_max(:,:)) 
       ENDDO

       ! Litter pool (gC m-2) *  (m2 m-2) 
       DO ilit = 1,nlitt
          DO ilev = 1,nlevs
             pool_end(:,:,iele) = pool_end(:,:,iele) + &
                  litter(:,ilit,:,ilev,iele) * veget_max(:,:)
          ENDDO
       ENDDO

       ! Soil carbon (gC m-2) *  (m2 m-2)
       DO icarb = 1,ncarb
          pool_end(:,:,iele) = pool_end(:,:,iele) + &
               carbon(:,icarb,:) * veget_max(:,:)
       ENDDO   
    ENDDO

    !! 4.2 Calculate components of the mass balance
    check_intern(:,:,iatm2land,icarbon) = orphan_flux_local(:,:,ico2bm,icarbon) * &
         orphan_flux_local(:,:,ivegnew,icarbon)
    check_intern(:,:,iland2atm,icarbon) = -un * zero
    check_intern(:,:,ilat2out,icarbon) = zero
    check_intern(:,:,ilat2in,icarbon) = -un * zero
    check_intern(:,:,ipoolchange,icarbon) = -un * (pool_end(:,:,icarbon) - pool_start(:,:,icarbon))
    closure_intern = zero
    DO imbc = 1,nmbcomp
       closure_intern(:,:,icarbon) = closure_intern(:,:,icarbon) + check_intern(:,:,imbc,icarbon)
    ENDDO

    ! Write outcome.  Need to some over all PFTs here because vegetation can
    ! move between them when an age class dies
    DO ipts=1,npts
       IF(ABS(SUM(closure_intern(ipts,:,icarbon))) .LE. min_stomate)THEN
          IF (ld_massbal) WRITE(numout,*) 'Mass balance closure in gap_clean'
       ELSE
          WRITE(numout,*) 'Error: mass balance is not closed in gap_clean',&
               SUM(closure_intern(ipts,:,icarbon))
          DO ivm=1,nvm
             WRITE(numout,*) '   ipts,ivm; ', ipts,ivm
             WRITE(numout,*) '   Difference is, ', closure_intern(ipts,ivm,icarbon)
             WRITE(numout,*) '   pool_end,pool_start: ', &
                  pool_end(ipts,ivm,icarbon), pool_start(ipts,ivm,icarbon)
             WRITE(numout,*) '   orphan_fluxes: ', &
                  orphan_flux_local(ipts,ivm,ico2bm,icarbon),&
                  orphan_flux_local(ipts,ivm,ivegnew,icarbon)
             WRITE(numout,*) '   veget_max: ', veget_max(ipts,ivm)
          ENDDO
          IF(ld_stop)THEN
             CALL ipslerr_p (3,'gap_clean', 'Mass balance error.','','')
          ENDIF
       ENDIF
    ENDDO

    !++++ CHECK+++++
    ! Here we should merge orphan_flux_local with the real orphan_flux passed out of this routine
    ! Right now we get an error in stomate_lpj and stomate after age classes migrate because
    ! we have not done this.  We get no error here, though.
    !+++++++++++++++


    IF (bavard.GE.2) WRITE(numout,*) 'Leaving gap_clean'

  END SUBROUTINE gap_clean

END MODULE stomate_kill
